import'reflect-metadata';import {ClientTypes}from'@common-stack/client-core';import {ScopedContainer}from'@common-stack/client-core/lib/connector/ScopedContainer.js';import {merge}from'lodash-es';import features from'../modules.js';import {UtilityClass,logger}from'../utils/index.js';import {createApolloClient}from'./base-apollo-client.js';import {config}from'./browser-env-config.js';import {getGlobalVariable}from'@common-stack/core';const utility = new UtilityClass(features);
ScopedContainer.registerGlobalDependencies((container) => {
    container.bind(ClientTypes.Logger).toConstantValue(logger);
    container.bind(ClientTypes.UtilityClass).toConstantValue(utility);
});
const createClientContainer = (req, res) => {
    logger.debug('Calling CreateClientContainer');
    const container = features.createContainers({});
    container
        .bind(ClientTypes.ApolloClient)
        .toDynamicValue((context) => new Error('Too early to bind ApolloClient'))
        .inRequestScope();
    container
        .bind(ClientTypes.ApolloClientFactory)
        .toDynamicValue((context) => () => {
        const newClient = container.get(ClientTypes.ApolloClient);
        return newClient;
    });
    const services = merge({ container: container }, ...features.createServiceFunc.map((serviceFunc) => serviceFunc(container)));
    const clientState = features.getStateParams({
        resolverContex: () => services,
        // container: container,
        requestResponsePair: {
            req,
            res,
        },
    });
    const { apolloClient, cache } = createApolloClient({
        httpGraphqlURL: config.GRAPHQL_URL,
        httpLocalGraphqlURL: config.LOCAL_GRAPHQL_URL,
        isDev: process.env.NODE_ENV === 'development',
        isDebug: getGlobalVariable('__DEBUGGING__'),
        isSSR: getGlobalVariable('__SSR__'),
        scope: typeof window !== 'undefined' ? 'browser' : 'server',
        clientState,
        getDataIdFromObject: (result) => features.getDataIdFromObject(result),
        initialState: typeof window !== 'undefined' ? window?.__APOLLO_STATE__ : undefined,
        logger,
    });
    if (!container.isBound(ClientTypes.InMemoryCache)) {
        container
            .bind(ClientTypes.InMemoryCache)
            .toDynamicValue((context) => cache)
            .inRequestScope();
    }
    container
        .rebind(ClientTypes.ApolloClient)
        .toDynamicValue((context) => apolloClient)
        .inRequestScope();
    // const services = serviceFunc();
    const serviceFunc = () => services;
    apolloClient.container = services;
    const clientService = {
        container: container,
        apolloClient,
        serviceFunc,
        logger,
    };
    // @ts-ignore
    if (import.meta.hot) {
        //@ts-ignore
        import.meta.hot.dispose(() => {
            // Force Apollo to fetch the latest data from the server
            delete window.__APOLLO_STATE__;
        });
    }
    if (typeof window !== 'undefined') {
        //@ts-ignore
        window.__CLIENT_SERVICE__ = clientService;
    }
    return clientService;
};export{createClientContainer};